/*
 * Worldvisions Weaver Software:
 *   Copyright (C) 1998, 1999 Worldvisions Computer Technology, Inc.
 *
 * The UDP-related parts of Fast Forward (UDPListener, UDPMap, etc).
 *
 */

#include "wvfastfwd.h"
#include "wvffudp.h"


unsigned WvHash( const WvFF_UDPMap& u )
/*************************************/
{
    return( ::WvHash( u.remote ) );
}


///////////////////////////////////////////////////////////
// WvFF_UDPMap stuff
///////////////////////////////////////////////////////////

bool WvFF_UDPMap::operator== ( const WvFF_UDPMap& u2 )
/****************************************************/
// blargh.  If WvIPPortAddr::operator== would compare port numbers, all of
// this would be unnecessary.
{
    return( remote == u2.remote && remote.port == u2.remote.port );
}


///////////////////////////////////////////////////////////
// WvFF_UDPListener stuff
///////////////////////////////////////////////////////////

WvFF_UDPListener::WvFF_UDPListener( const WvIPPortAddr& local,
				    WvFastForward *	forward,
				    const char *	rem )
/***********************************************************/
: WvUDPStream( local, WvIPAddr() ), fwd( *forward ), map( 7 ),
  fwd_to( rem )
{
    setcallback( udp_callback, this );
    last_checked = time( NULL );
}

WvFF_UDPListener::~WvFF_UDPListener()
/***********************************/
// When a UDP listener shuts down, we need to remove all its mappings
// (ie - ports received from in the last little while that we're hanging on
// to in case a reply needs to be sent back) from Fast Forward's stream list.
{
    // Oct 6 -- When Fast Forward is shutting down, we don't want to clear out
    // the UDP entries from WvFastForward's udp_streams list... The
    // WvFastForward class does that when IT destructs.  I wonder how long
    // THAT bug (causes segfault) was there...
    if( fwd.shutting_down )
    	return;

    WvFF_UDPMapTable::Iter iter( map );

    for( iter.rewind(); iter.next(); ) {
    	WvFF_UDPMap& m = iter;
    	fwd.get_udp().unlink( m.bounce_stream );
    }
}

void WvFF_UDPListener::udp_callback( WvStream& s, void * userdata )
/*****************************************************************/
// Called when a UDP packet is received on whatever port this listener is
// listening on.
{
    WvFF_UDPListener& lstnr = *(WvFF_UDPListener *) userdata;

    if( !lstnr.select( 0 ) )
    	return;

    char	  buf[ 10240 ];
    size_t	  len = lstnr.read( buf, sizeof( buf ) );
    WvIPPortAddr& rem = *(WvIPPortAddr *) lstnr.src();
    WvFF_UDPMap * map = lstnr.map[ WvFF_UDPMap( rem ) ];

    if( !map ) {
    	WvIPPortAddr destaddr;
	char *	     err_buf;
    	if( make_port_addr( destaddr, lstnr.fwd_to, &err_buf ) == false ) {
    	    lstnr.fwd.err( err_buf );
    	    return;
    	}
    	WvStream * to = new WvUDPStream( WvIPAddr(), destaddr );
    	map = new WvFF_UDPMap( &lstnr, rem, to );
    	to->setcallback( udp_callback2, map );
    	lstnr.fwd.get_udp().append( to, false );
    	lstnr.map.add( map, true );
    	lstnr.fwd.log( "UDP: forwarding %s to %s.\n", rem, destaddr );
    }

    map->bounce_stream->write( buf, len );
    map->last_used = time( NULL );

    // delete idle stream forwarders
    if( lstnr.last_checked + 5 < time( NULL ) ) {
    	time_t now = lstnr.last_checked = time( NULL );

    	WvFF_UDPMapTable::Iter	iter( lstnr.map );
    	for( iter.rewind(); iter.next(); ) {
    	    WvFF_UDPMap& map = iter;

    	    if( map.last_used + 30 < now ) {
    	    	// idle
    	    	lstnr.fwd.get_udp().unlink( map.bounce_stream );
    	    	lstnr.map.remove( &map );
    	    	iter.rewind();
    	    }
    	}
    }
}

void WvFF_UDPListener::udp_callback2( WvStream& bouncer, void * userdata )
/************************************************************************/
// Called when something is received as a reply to a packet we've forwarded.
{
    if( !bouncer.select( 0 ) )
    	return;

    char 	 buf[ 10240 ];
    WvFF_UDPMap& map = *(WvFF_UDPMap *) userdata;
    size_t	 len = bouncer.read( buf, sizeof( buf ) );

    if( len ) {
    	map.lstnr->setdest( map.remote );
    	map.lstnr->write( buf, len );
    }

    map.last_used = time( NULL );
}

