/*
 * Worldvisions Weaver Software:
 *   Copyright (C) 1998, 1999 Worldvisions Computer Technology, Inc.
 *
 * The UDP-related parts of Fast Forward (UDPListener, UDPMap, etc).
 *
 */

#ifndef __WVFFUDP_H
#define __WVFFUDP_H

#include "wvhashtable.h"
#include "wvudp.h"

#include <time.h>

// forward declarations
class WvFF_UDPListener;
class WvFastForward;

struct WvFF_UDPMap
/****************/
// When forwarding UDP packets, we need to remember who the packet came from,
// so that any replies we receive can be forwarded back to the original port.
// WvFF_UDPMap keeps track of all the required information.
{
    WvFF_UDPListener *	lstnr;
    WvIPPortAddr	remote;
    WvStream *		bounce_stream;
    time_t		last_used;

    WvFF_UDPMap( WvFF_UDPListener *  l,
    		 const WvIPPortAddr& rem,
    		 WvStream *	     bounce )
    	: lstnr( l ), remote( rem ), bounce_stream( bounce )
    	{ last_used = time( NULL ); }

    // mostly needed for hackish operator==
    WvFF_UDPMap( const WvIPPortAddr& rem )
    	: remote( rem ), bounce_stream( NULL )
    	{ }

    ~WvFF_UDPMap()
    	{ if( bounce_stream ) delete bounce_stream; }

    bool operator== ( const WvFF_UDPMap& u );
};

unsigned WvHash( const WvFF_UDPMap& u );
DeclareWvTable( WvFF_UDPMap );

class WvFF_UDPListener : public WvUDPStream
/*****************************************/
{
public:
    WvFF_UDPListener( const WvIPPortAddr& local,
                      WvFastForward *	  forward,
    		      const char *	  rem );
    virtual ~WvFF_UDPListener();

private:
    WvFastForward& 	fwd;
    WvFF_UDPMapTable	map;
    WvString		fwd_to;
    time_t		last_checked;

    static Callback udp_callback;
    static Callback udp_callback2;
};

#endif __WVFFUDP_H
